##############################################################################
# balloon.tcl - procedures used by balloon help
#
# Copyright (C) 1996-1997 Stewart Allen
# 
# This is part of vtcl source code
# Adapted for general purpose by 
# Daniel Roche <daniel.roche@bigfoot.com>
# thanks to D. Richard Hipp for the multi-headed display fix
# version 1.2 ( Sep 21 2000 ) 
#
# Modified 15-Nov-2000 by Andres Garcia <fandom@retemail.es>
#     - Wrapped it up in a namespace.
#     - Put more horizontal distance bewteen the cursor and
#       the balloon.
#     - Added a workaround, in some cases, I think due to
#       KDE2 final, you get Enter and Leave events while
#       you are in a widget. The 'Motion' variable prevents
#       it from making the balloons go in and out.
#       Kde2.1 beta has solved this problem, so I will soon
#       take this out.
#     - Added the 'delete_balloon' procedure to unbind a 
#       widget to a balloon.
#
# Modified 14-Jan-01 by Andres Garcia <fandom@retemail.es>
#     - Added a <destroy> binding to the Bulle class so
#       that if a widget that has a ballon gets destroyed
#       the ballon won't survive it.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################
#

namespace eval BalloonHelp {

set Motion 0

bind Bulle <Enter> {
    set BalloonHelp::Bulle(set)   0
    set BalloonHelp::Bulle(first) 1
    if {$BalloonHelp::Motion==0} {
        set BalloonHelp::Bulle(id) \
                [after 500 {BalloonHelp::balloon %W %X %Y}]
    }  
}

bind Bulle <Button> {
    set BalloonHelp::Bulle(first) 0
    BalloonHelp::kill_balloon
}

bind Bulle <Leave> {
    if {$BalloonHelp::Motion==2} {
        set BalloonHelp::Bulle(first) 0
        BalloonHelp::kill_balloon
        set BalloonHelp::Motion 0
    } elseif {$BalloonHelp::Motion==0} {
        after cancel $BalloonHelp::Bulle(id)
    }
    if {[winfo viewable %W]==0} {
        catch {destroy .balloon}
    }
}

bind Bulle <Destroy> {
    BalloonHelp::delete_balloon %W
}

bind Bulle <Motion> {
    if {$BalloonHelp::Motion==0} {
        if {$BalloonHelp::Bulle(set) == 0} {
            after cancel $BalloonHelp::Bulle(id)
            set BalloonHelp::Bulle(id) \
                    [after 500 {BalloonHelp::balloon %W %X %Y}]
        }
    } elseif {$BalloonHelp::Motion==1} {
        set BalloonHelp::Motion 2
    }
}

proc set_balloon {target message} {
    variable Bulle

    if {![info exists Bulle($target)]} {
        bindtags $target "[bindtags $target] Bulle"
    } else {
        if {![string match $Bulle($target) $message]} {
            catch {destroy .balloon}
        }
    }
    set Bulle($target) $message

   return
}

proc delete_balloon {target} {
    variable Bulle

    if {![info exists Bulle($target)]} {
        return
    }
    after cancel Bulle(id) 
    unset Bulle($target)
    set bindList   [bindtags $target]
    set bulleIndex [lsearch $bindList Bulle]
    set bindList   [lreplace $bindList $bulleIndex $bulleIndex]
    bindtags $target $bindList

    catch {destroy .balloon}

    return
}

proc kill_balloon {} {
    variable Bulle
    after cancel $Bulle(id)
    if {[winfo exists .balloon] == 1} {
        destroy .balloon
    }
    set Bulle(set) 0

    return
}

proc balloon {target {cx 0} {cy 0}} {
    variable Bulle
    variable Motion

    if {[catch {set Bulle($target)} message]} return

    if {$Bulle(first) == 1} {
        set Motion 1
        set Bulle(first) 2
        if { $cx == 0 && $cy == 0 } {
            set x [expr [winfo rootx $target] + ([winfo width $target]/2)]
            set y [expr [winfo rooty $target] + [winfo height $target] + 4]
        } else {
            set x [expr $cx + 8]
            set y [expr $cy + 4]
        }
        toplevel .balloon -bg black -screen [winfo screen $target]
        wm overrideredirect .balloon 1
        label .balloon.l \
                -text $message -relief flat \
                -bg #ffffaa -fg black -padx 2 -pady 0 -anchor w
        pack .balloon.l -side left -padx 1 -pady 1
        wm geometry .balloon +${x}+${y}
        set Bulle(set) 1
    }
    return
}

}
